@@ -1,20 +1,20 @@
-/* $Id: audio-tags-page.c 2017 2006-09-11 09:41:38Z jannis $ */
+/* $Id$ */
 /*-
  * Copyright (c) 2006 Jannis Pohlmann <jannis@xfce.org>
  *
- * This program is free software; you can redistribute it and/or 
- * modify it under the terms of the GNU General Public License as 
- * published by the Free Software Foundation; either version 2 of the 
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 2 of the
  * License, or (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful, but 
- * WITHOUT ANY WARRANTY; without even the implied warranty of 
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License 
- * along with this program; if not, write to the Free Software 
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
  * MA 02111-1307 USA.
  */
 
@@ -32,14 +32,13 @@
 #include <taglib/tag_c.h>
 
 #include <exo/exo.h>
-#include <thunar-vfs/thunar-vfs.h>
 
 #include <audio-tags-page.h>
 
 
 
 /* Property identifiers */
-enum 
+enum
 {
   PROP_0,
   PROP_FILE,
@@ -56,8 +55,6 @@ enum
 
 
 
-static void     audio_tags_page_class_init          (AudioTagsPageClass *klass);
-static void     audio_tags_page_init                (AudioTagsPage      *page);
 static void     audio_tags_page_finalize            (GObject            *object);
 static void     audio_tags_page_get_property        (GObject            *object,
                                                      guint               prop_id,
@@ -74,7 +71,7 @@ static void     audio_tags_page_taglib_file_changed (TagLib_File        *taglib_
 static gboolean audio_tags_page_activate            (AudioTagsPage      *page);
 static gboolean audio_tags_page_info_activate       (GtkAction          *action,
                                                      AudioTagsPage      *page);
-static gboolean audio_tags_page_load_tags           (AudioTagsPage      *page);
+static gboolean audio_tags_page_load_tags           (gpointer            data);
 
 
 
@@ -90,13 +87,12 @@ struct _AudioTagsPage
   ThunarxPropertyPage __parent__;
 
   /* Widgets */
-  GtkTooltips     *tooltips;
   GtkWidget       *table;
   GtkWidget       *save_button;
   GtkWidget       *info_button;
 
   /* Timeouts */
-  guint            changed_timeout;
+  guint            changed_idle;
 
   /* Properties */
   ThunarxFileInfo *file;
@@ -111,6 +107,9 @@ struct _AudioTagsPage
 
   /* <private> */
   GtkActionGroup  *action_group;
+
+  /* garbage collector of files */
+  GSList           *files_gc;
 };
 
 
@@ -169,7 +168,7 @@ audio_tags_page_class_init (AudioTagsPageClass *klass)
                                                         999,
                                                         1,
                                                         G_PARAM_READWRITE));
-  
+
   /**
    * TagRenamerPropertiesPage:artist:
    *
@@ -249,7 +248,7 @@ audio_tags_page_class_init (AudioTagsPageClass *klass)
                                                         9999,
                                                         2006,
                                                         G_PARAM_READWRITE));
-  
+
   /**
    * TagRenamerPropertiesPage:show-save-button:
    *
@@ -278,143 +277,7 @@ audio_tags_page_init (AudioTagsPage *page)
   GtkWidget *spin;
   GtkWidget *toplevel;
   GtkAction *action;
-  int i;
-
-  gtk_container_set_border_width (GTK_CONTAINER (page), 8);
-
-  /* Allocate the shared tooltips */
-  page->tooltips = gtk_tooltips_new ();
-  exo_gtk_object_ref_sink (GTK_OBJECT (page->tooltips));
-
-  /* Main container */
-  vbox = gtk_vbox_new (FALSE, 8);
-  gtk_container_add (GTK_CONTAINER (page), vbox);
-  gtk_widget_show (vbox);
-
-  /* Tag editor layout table */
-  page->table = gtk_table_new (7, 4, FALSE);
-  gtk_table_set_row_spacings (GTK_TABLE (page->table), 6);
-  gtk_table_set_col_spacings (GTK_TABLE (page->table), 12);
-  gtk_container_add (GTK_CONTAINER (vbox), page->table);
-  gtk_container_set_border_width (GTK_CONTAINER (page->table), 12);
-  gtk_widget_show (page->table);
-
-  /* Track label */
-  label = gtk_label_new ("");
-  gtk_misc_set_alignment (GTK_MISC (label), 1.0f, 0.5f);
-  gtk_label_set_markup (GTK_LABEL (label), _("<b>Track:</b>"));
-  gtk_table_attach (GTK_TABLE (page->table), label, 0, 1, 0, 1, GTK_FILL, 0, 0, 0);
-  gtk_widget_show (label);
-
-  /* Track spin button alignment */
-  alignment = gtk_alignment_new (0.0, 0.5, 0.0, 0.0);
-  gtk_table_attach (GTK_TABLE (page->table), alignment, 1, 2, 0, 1, GTK_FILL, 0, 0, 0);
-  gtk_widget_show (alignment);
-
-  /* Track adjustment */
-  adjustment = gtk_adjustment_new (0, 0, 999, 1, 5, 10);
-
-  /* Track spin button */
-  spin = gtk_spin_button_new (GTK_ADJUSTMENT (adjustment), 1, 0);
-  exo_mutual_binding_new (G_OBJECT (adjustment), "value", G_OBJECT (page), "track");
-  gtk_tooltips_set_tip (page->tooltips, spin, _("Enter the track number here."), NULL);
-  gtk_container_add (GTK_CONTAINER (alignment), spin);
-  gtk_label_set_mnemonic_widget (GTK_LABEL (label), spin);
-  gtk_widget_show (spin);
-  
-  /* Year label */
-  label = gtk_label_new ("");
-  gtk_misc_set_alignment (GTK_MISC (label), 1.0f, 0.5f);
-  gtk_label_set_markup (GTK_LABEL (label), _("<b>Year:</b>"));
-  gtk_table_attach (GTK_TABLE (page->table), label, 2, 3, 0, 1, GTK_FILL, 0, 0, 0);
-  gtk_widget_show (label);
-
-  /* Year spin button alignment */
-  alignment = gtk_alignment_new (0.0, 0.5, 0.0, 0.0);
-  gtk_table_attach (GTK_TABLE (page->table), alignment, 3, 4, 0, 1, GTK_FILL, 0, 0, 0);
-  gtk_widget_show (alignment);
-
-  /* Year spin button range description */
-  adjustment = gtk_adjustment_new (2006, 1700, 9999, 1, 10, 100);
-
-  /* Year spin button */
-  spin = gtk_spin_button_new (GTK_ADJUSTMENT (adjustment), 1, 0);
-  exo_mutual_binding_new (G_OBJECT (adjustment), "value", G_OBJECT (page), "year");
-  gtk_tooltips_set_tip (page->tooltips, spin, _("Enter the release year here."), NULL);
-  gtk_container_add (GTK_CONTAINER (alignment), spin);
-  gtk_label_set_mnemonic_widget (GTK_LABEL (label), spin);
-  gtk_widget_show (spin);
-  
-  /* Artist label */
-  label = gtk_label_new ("");
-  gtk_misc_set_alignment (GTK_MISC (label), 1.0f, 0.5f);
-  gtk_label_set_markup (GTK_LABEL (label), _("<b>Artist:</b>"));
-  gtk_table_attach (GTK_TABLE (page->table), label, 0, 1, 1, 2, GTK_FILL, 0, 0, 0);
-  gtk_widget_show (label);
-
-  /* Artist entry */
-  entry = gtk_entry_new ();
-  exo_mutual_binding_new (G_OBJECT (entry), "text", G_OBJECT (page), "artist");
-  gtk_tooltips_set_tip (page->tooltips, entry, _("Enter the name of the artist or author of this file here."), NULL);
-  gtk_table_attach (GTK_TABLE (page->table), entry, 1, 4, 1, 2, GTK_EXPAND | GTK_FILL, 0, 0, 0);
-  gtk_label_set_mnemonic_widget (GTK_LABEL (label), entry);
-  gtk_widget_show (entry);
-  
-  /* Title label */
-  label = gtk_label_new ("");
-  gtk_misc_set_alignment (GTK_MISC (label), 1.0f, 0.5f);
-  gtk_label_set_markup (GTK_LABEL (label), _("<b>Title:</b>"));
-  gtk_table_attach (GTK_TABLE (page->table), label, 0, 1, 2, 3, GTK_FILL, 0, 0, 0);
-  gtk_widget_show (label);
-
-  /* Title entry */
-  entry = gtk_entry_new ();
-  exo_mutual_binding_new (G_OBJECT (entry), "text", G_OBJECT (page), "title");
-  gtk_tooltips_set_tip (page->tooltips, entry, _("Enter the song title here."), NULL);
-  gtk_table_attach (GTK_TABLE (page->table), entry, 1, 4, 2, 3, GTK_EXPAND | GTK_FILL, 0, 0, 0);
-  gtk_widget_show (entry);
-
-  /* Album label */
-  label = gtk_label_new ("");
-  gtk_misc_set_alignment (GTK_MISC (label), 1.0f, 0.5f);
-  gtk_label_set_markup (GTK_LABEL (label), _("<b>Album:</b>"));
-  gtk_table_attach (GTK_TABLE (page->table), label, 0, 1, 3, 4, GTK_FILL, 0, 0, 0);
-  gtk_widget_show (label);
-
-  /* Album entry */
-  entry = gtk_entry_new ();
-  exo_mutual_binding_new (G_OBJECT (entry), "text", G_OBJECT (page), "album");
-  gtk_tooltips_set_tip (page->tooltips, entry, _("Enter the album/record title here."), NULL);
-  gtk_table_attach (GTK_TABLE (page->table), entry, 1, 4, 3, 4, GTK_EXPAND | GTK_FILL, 0, 0, 0);
-  gtk_widget_show (entry);
-  
-  /* Comment label */
-  label = gtk_label_new ("");
-  gtk_misc_set_alignment (GTK_MISC (label), 1.0f, 0.5f);
-  gtk_label_set_markup (GTK_LABEL (label), _("<b>Comment:</b>"));
-  gtk_table_attach (GTK_TABLE (page->table), label, 0, 1, 4, 5, GTK_FILL, 0, 0, 0);
-  gtk_widget_show (label);
-
-  /* Comment entry */
-  entry = gtk_entry_new ();
-  exo_mutual_binding_new (G_OBJECT (entry), "text", G_OBJECT (page), "comment");
-  gtk_tooltips_set_tip (page->tooltips, entry, _("Enter your comments here."), NULL);
-  gtk_table_attach (GTK_TABLE (page->table), entry, 1, 4, 4, 5, GTK_EXPAND | GTK_FILL, 0, 0, 0);
-  gtk_widget_show (entry);
-
-  /* Genre label */
-  label = gtk_label_new ("");
-  gtk_misc_set_alignment (GTK_MISC (label), 1.0f, 0.5f);
-  gtk_label_set_markup (GTK_LABEL (label), _("<b>Genre:</b>"));
-  gtk_table_attach (GTK_TABLE (page->table), label, 0, 1, 5, 6, GTK_FILL, 0, 0, 0);
-  gtk_widget_show (label);
-
-  /* Genre combo box */
-  combo = gtk_combo_box_entry_new_text ();
-  exo_mutual_binding_new (G_OBJECT (GTK_BIN (combo)->child), "text", G_OBJECT (page), "genre");
-  gtk_tooltips_set_tip (page->tooltips, combo, _("Select or enter the genre of this song here."), NULL);
-  gtk_table_attach (GTK_TABLE (page->table), combo, 1, 4, 5, 6, GTK_FILL, 0, 0, 0);
-  gtk_widget_show (combo);
+  guint      i;
 
   /* Default genre list */
   static const gchar *genres[] = {
@@ -444,7 +307,139 @@ audio_tags_page_init (AudioTagsPage *page)
     "Techno", "Techno-Industrial", "Terror", "Thrash Metal", "Top 40", "Trailer",
   };
 
-  for (i=0; i<G_N_ELEMENTS (genres); i++) 
+  gtk_container_set_border_width (GTK_CONTAINER (page), 8);
+
+  /* Main container */
+  vbox = gtk_vbox_new (FALSE, 8);
+  gtk_container_add (GTK_CONTAINER (page), vbox);
+  gtk_widget_show (vbox);
+
+  /* Tag editor layout table */
+  page->table = gtk_table_new (7, 4, FALSE);
+  gtk_table_set_row_spacings (GTK_TABLE (page->table), 6);
+  gtk_table_set_col_spacings (GTK_TABLE (page->table), 12);
+  gtk_container_add (GTK_CONTAINER (vbox), page->table);
+  gtk_container_set_border_width (GTK_CONTAINER (page->table), 12);
+  gtk_widget_show (page->table);
+
+  /* Track label */
+  label = gtk_label_new ("");
+  gtk_misc_set_alignment (GTK_MISC (label), 1.0f, 0.5f);
+  gtk_label_set_markup (GTK_LABEL (label), _("<b>Track:</b>"));
+  gtk_table_attach (GTK_TABLE (page->table), label, 0, 1, 0, 1, GTK_FILL, 0, 0, 0);
+  gtk_widget_show (label);
+
+  /* Track spin button alignment */
+  alignment = gtk_alignment_new (0.0, 0.5, 0.0, 0.0);
+  gtk_table_attach (GTK_TABLE (page->table), alignment, 1, 2, 0, 1, GTK_FILL, 0, 0, 0);
+  gtk_widget_show (alignment);
+
+  /* Track adjustment */
+  adjustment = gtk_adjustment_new (0, 0, 999, 1, 5, 0);
+
+  /* Track spin button */
+  spin = gtk_spin_button_new (GTK_ADJUSTMENT (adjustment), 0.0, 0);
+  exo_mutual_binding_new (G_OBJECT (adjustment), "value", G_OBJECT (page), "track");
+  gtk_widget_set_tooltip_text (GTK_WIDGET (spin), _("Enter the track number here."));
+  gtk_container_add (GTK_CONTAINER (alignment), spin);
+  gtk_label_set_mnemonic_widget (GTK_LABEL (label), spin);
+  gtk_widget_show (spin);
+
+  /* Year label */
+  label = gtk_label_new ("");
+  gtk_misc_set_alignment (GTK_MISC (label), 1.0f, 0.5f);
+  gtk_label_set_markup (GTK_LABEL (label), _("<b>Year:</b>"));
+  gtk_table_attach (GTK_TABLE (page->table), label, 2, 3, 0, 1, GTK_FILL, 0, 0, 0);
+  gtk_widget_show (label);
+
+  /* Year spin button alignment */
+  alignment = gtk_alignment_new (0.0, 0.5, 0.0, 0.0);
+  gtk_table_attach (GTK_TABLE (page->table), alignment, 3, 4, 0, 1, GTK_FILL, 0, 0, 0);
+  gtk_widget_show (alignment);
+
+  /* Year spin button range description */
+  adjustment = gtk_adjustment_new (2006, 1700, 9999, 1, 10, 0);
+
+  /* Year spin button */
+  spin = gtk_spin_button_new (GTK_ADJUSTMENT (adjustment), 1, 0);
+  exo_mutual_binding_new (G_OBJECT (adjustment), "value", G_OBJECT (page), "year");
+  gtk_widget_set_tooltip_text (GTK_WIDGET (spin), _("Enter the release year here."));
+  gtk_container_add (GTK_CONTAINER (alignment), spin);
+  gtk_label_set_mnemonic_widget (GTK_LABEL (label), spin);
+  gtk_widget_show (spin);
+
+  /* Artist label */
+  label = gtk_label_new ("");
+  gtk_misc_set_alignment (GTK_MISC (label), 1.0f, 0.5f);
+  gtk_label_set_markup (GTK_LABEL (label), _("<b>Artist:</b>"));
+  gtk_table_attach (GTK_TABLE (page->table), label, 0, 1, 1, 2, GTK_FILL, 0, 0, 0);
+  gtk_widget_show (label);
+
+  /* Artist entry */
+  entry = gtk_entry_new ();
+  exo_mutual_binding_new (G_OBJECT (entry), "text", G_OBJECT (page), "artist");
+  gtk_widget_set_tooltip_text (GTK_WIDGET (entry), _("Enter the name of the artist or author of this file here."));
+  gtk_table_attach (GTK_TABLE (page->table), entry, 1, 4, 1, 2, GTK_EXPAND | GTK_FILL, 0, 0, 0);
+  gtk_label_set_mnemonic_widget (GTK_LABEL (label), entry);
+  gtk_widget_show (entry);
+
+  /* Title label */
+  label = gtk_label_new ("");
+  gtk_misc_set_alignment (GTK_MISC (label), 1.0f, 0.5f);
+  gtk_label_set_markup (GTK_LABEL (label), _("<b>Title:</b>"));
+  gtk_table_attach (GTK_TABLE (page->table), label, 0, 1, 2, 3, GTK_FILL, 0, 0, 0);
+  gtk_widget_show (label);
+
+  /* Title entry */
+  entry = gtk_entry_new ();
+  exo_mutual_binding_new (G_OBJECT (entry), "text", G_OBJECT (page), "title");
+  gtk_widget_set_tooltip_text (GTK_WIDGET (entry), _("Enter the song title here."));
+  gtk_table_attach (GTK_TABLE (page->table), entry, 1, 4, 2, 3, GTK_EXPAND | GTK_FILL, 0, 0, 0);
+  gtk_widget_show (entry);
+
+  /* Album label */
+  label = gtk_label_new ("");
+  gtk_misc_set_alignment (GTK_MISC (label), 1.0f, 0.5f);
+  gtk_label_set_markup (GTK_LABEL (label), _("<b>Album:</b>"));
+  gtk_table_attach (GTK_TABLE (page->table), label, 0, 1, 3, 4, GTK_FILL, 0, 0, 0);
+  gtk_widget_show (label);
+
+  /* Album entry */
+  entry = gtk_entry_new ();
+  exo_mutual_binding_new (G_OBJECT (entry), "text", G_OBJECT (page), "album");
+  gtk_widget_set_tooltip_text (GTK_WIDGET (entry), _("Enter the album/record title here."));
+  gtk_table_attach (GTK_TABLE (page->table), entry, 1, 4, 3, 4, GTK_EXPAND | GTK_FILL, 0, 0, 0);
+  gtk_widget_show (entry);
+
+  /* Comment label */
+  label = gtk_label_new ("");
+  gtk_misc_set_alignment (GTK_MISC (label), 1.0f, 0.5f);
+  gtk_label_set_markup (GTK_LABEL (label), _("<b>Comment:</b>"));
+  gtk_table_attach (GTK_TABLE (page->table), label, 0, 1, 4, 5, GTK_FILL, 0, 0, 0);
+  gtk_widget_show (label);
+
+  /* Comment entry */
+  entry = gtk_entry_new ();
+  exo_mutual_binding_new (G_OBJECT (entry), "text", G_OBJECT (page), "comment");
+  gtk_widget_set_tooltip_text (GTK_WIDGET (entry), _("Enter your comments here."));
+  gtk_table_attach (GTK_TABLE (page->table), entry, 1, 4, 4, 5, GTK_EXPAND | GTK_FILL, 0, 0, 0);
+  gtk_widget_show (entry);
+
+  /* Genre label */
+  label = gtk_label_new ("");
+  gtk_misc_set_alignment (GTK_MISC (label), 1.0f, 0.5f);
+  gtk_label_set_markup (GTK_LABEL (label), _("<b>Genre:</b>"));
+  gtk_table_attach (GTK_TABLE (page->table), label, 0, 1, 5, 6, GTK_FILL, 0, 0, 0);
+  gtk_widget_show (label);
+
+  /* Genre combo box */
+  combo = gtk_combo_box_entry_new_text ();
+  exo_mutual_binding_new (G_OBJECT (GTK_BIN (combo)->child), "text", G_OBJECT (page), "genre");
+  gtk_widget_set_tooltip_text (GTK_WIDGET (combo), _("Select or enter the genre of this song here."));
+  gtk_table_attach (GTK_TABLE (page->table), combo, 1, 4, 5, 6, GTK_FILL, 0, 0, 0);
+  gtk_widget_show (combo);
+
+  for (i=0; i<G_N_ELEMENTS (genres); i++)
     gtk_combo_box_append_text (GTK_COMBO_BOX (combo), genres[i]);
 
   /* Create action group for the page */
@@ -453,7 +448,7 @@ audio_tags_page_init (AudioTagsPage *page)
   /* Create and add the save action */
   action = gtk_action_new ("save", _("_Save"), _("Save audio tags."), GTK_STOCK_SAVE);
   gtk_action_group_add_action (page->action_group, action);
-  
+
   /* Connect to save action */
   g_signal_connect_swapped (G_OBJECT (action), "activate", G_CALLBACK (audio_tags_page_activate), page);
 
@@ -468,7 +463,7 @@ audio_tags_page_init (AudioTagsPage *page)
 
   /* Connect to the info action */
   g_signal_connect (G_OBJECT (action), "activate", G_CALLBACK (audio_tags_page_info_activate), page);
-  
+
   /* Reset the save button */
   audio_tags_page_set_show_save_button (page, FALSE);
 }
@@ -481,21 +476,25 @@ audio_tags_page_finalize (GObject *object)
   AudioTagsPage *page = AUDIO_TAGS_PAGE (object);
 
   /* Destroy action group */
-  if (G_LIKELY (page->action_group != NULL)) 
+  if (G_LIKELY (page->action_group != NULL))
     g_object_unref (G_OBJECT (page->action_group));
 
   /* TODO Do I need to free the actions explicitely? */
 
-  /* Unregister the changed_timeout */
-  if (G_UNLIKELY (page->changed_timeout != 0))
-    g_source_remove (page->changed_timeout);
-  
+  /* Unregister the changed_idle */
+  if (G_UNLIKELY (page->changed_idle != 0))
+    g_source_remove (page->changed_idle);
+
   /* Free file reference */
   audio_tags_page_set_file (page, NULL);
 
   /* Free taglib file reference */
   audio_tags_page_set_taglib_file (page, NULL);
 
+  /* Cleanup garbage */
+  g_slist_foreach (page->files_gc, (GFunc) taglib_file_free, NULL);
+  g_slist_free (page->files_gc);
+
   /* Free strings */
   if (G_LIKELY (page->artist != NULL))
     g_free (page->artist);
@@ -512,9 +511,6 @@ audio_tags_page_finalize (GObject *object)
   if (G_LIKELY (page->genre != NULL))
     g_free (page->genre);
 
-  /* Release the tooltips */
-  g_object_unref (G_OBJECT (page->tooltips));
-
   (*G_OBJECT_CLASS (audio_tags_page_parent_class)->finalize) (object);
 }
 
@@ -536,7 +532,7 @@ AudioTagsPage*
 audio_tags_page_new_with_save_button (void)
 {
   AudioTagsPage *page = audio_tags_page_new ();
-  
+
   /* Add save button */
   audio_tags_page_set_show_save_button (page, TRUE);
 
@@ -546,7 +542,7 @@ audio_tags_page_new_with_save_button (void)
 
 
 GtkWidget*
-audio_tags_page_dialog_new (GtkWindow *window, 
+audio_tags_page_dialog_new (GtkWindow *window,
                             ThunarxFileInfo *file)
 {
   GtkWidget     *dialog;
@@ -559,12 +555,12 @@ audio_tags_page_dialog_new (GtkWindow *window,
 
   /* Set audio file */
   audio_tags_page_set_file (page, file);
-  
+
   /* Set up the dialog */
   dialog = gtk_dialog_new_with_buttons (_("Edit Tags"),
                                         window,
                                         GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
-                                        GTK_STOCK_CANCEL, 
+                                        GTK_STOCK_CANCEL,
                                         GTK_RESPONSE_CANCEL,
                                         NULL);
 
@@ -574,7 +570,7 @@ audio_tags_page_dialog_new (GtkWindow *window,
   /* Add page to the dialog */
   gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox), GTK_WIDGET (page));
   gtk_widget_show (GTK_WIDGET (page));
-  
+
   /* Create save button */
   button = gtk_button_new_from_stock (GTK_STOCK_SAVE);
 
@@ -631,7 +627,7 @@ audio_tags_page_get_property (GObject    *object,
     case PROP_GENRE:
       g_value_set_string (value, page->genre);
       break;
-      
+
     case PROP_YEAR:
       g_value_set_double (value, page->year);
       break;
@@ -665,7 +661,7 @@ audio_tags_page_set_property (GObject      *object,
     case PROP_TAGLIB_FILE:
       audio_tags_page_set_taglib_file (page, g_value_get_pointer (value));
       break;
-      
+
     case PROP_TRACK:
       page->track = g_value_get_double (value);
       break;
@@ -681,25 +677,25 @@ audio_tags_page_set_property (GObject      *object,
         g_free (page->title);
       page->title = g_strstrip (g_strdup (g_value_get_string (value)));
       break;
-      
+
     case PROP_ALBUM:
       if (G_LIKELY (page->album != NULL))
         g_free (page->album);
       page->album = g_strstrip (g_strdup (g_value_get_string (value)));
       break;
-      
+
     case PROP_COMMENT:
       if (G_LIKELY (page->comment != NULL))
         g_free (page->comment);
       page->comment = g_strstrip (g_strdup (g_value_get_string (value)));
       break;
-      
+
     case PROP_GENRE:
       if (G_LIKELY (page->genre != NULL))
         g_free (page->genre);
       page->genre = g_strstrip (g_strdup (g_value_get_string (value)));
       break;
-      
+
     case PROP_YEAR:
       page->year = g_value_get_double (value);
       break;
@@ -720,7 +716,7 @@ audio_tags_page_set_property (GObject      *object,
  * audio_tags_page_get_file:
  * @page : a #AudioTagsPage.
  *
- * Returns the current #ThunarxFileInfo 
+ * Returns the current #ThunarxFileInfo
  * for the @page.
  *
  * Return value: the file associated with this property page.
@@ -738,7 +734,7 @@ audio_tags_page_get_file (AudioTagsPage *page)
  * audio_tags_page_set_file:
  * @page : a #AudioTagsPage.
  * @file : a #ThunarxFileInfo
- * 
+ *
  * Sets the #ThunarxFileInfo for this @page.
  **/
 void
@@ -767,9 +763,9 @@ audio_tags_page_set_file (AudioTagsPage   *page,
     {
       /* Take a reference on the info file */
       g_object_ref (G_OBJECT (page->file));
-      
+
       audio_tags_page_file_changed (file, page);
-      
+
       g_signal_connect (G_OBJECT (file), "changed", G_CALLBACK (audio_tags_page_file_changed), page);
     }
 }
@@ -780,7 +776,7 @@ audio_tags_page_set_file (AudioTagsPage   *page,
  * audio_tags_page_get_taglib_file:
  * @page : a #AudioTagsPage.
  *
- * Returns the current #TagLib_File 
+ * Returns the current #TagLib_File
  * for the @page.
  *
  * Return value: the taglib file associated with this property page.
@@ -798,7 +794,7 @@ audio_tags_page_get_taglib_file (AudioTagsPage *page)
  * audio_tags_page_set_taglib_file:
  * @page : a #AudioTagsPage.
  * @file : a #TagLib_File
- * 
+ *
  * Sets the #TagLib_File for this @page.
  **/
 void
@@ -811,9 +807,10 @@ audio_tags_page_set_taglib_file (AudioTagsPage   *page,
   if (G_UNLIKELY (page->taglib_file == taglib_file))
     return;
 
-  /* Destroy the current object */
+  /* Freeing a file triggers a reload (taglib does that),
+   * so put it in the garbage collector */
   if (G_LIKELY (page->taglib_file != NULL))
-      taglib_file_free (page->taglib_file);
+    page->files_gc = g_slist_prepend (page->files_gc, page->taglib_file);
 
   /* Assign the value */
   page->taglib_file = taglib_file;
@@ -833,15 +830,8 @@ audio_tags_page_file_changed (ThunarxFileInfo *file,
   g_return_if_fail (IS_AUDIO_TAGS_PAGE (page));
   g_return_if_fail (page->file == file);
 
-  /* Temporarily reset the file attribute */
-  page->file = NULL;
-
-  /* Load tag information after 250 ms */
-  if (page->changed_timeout <= 0)
-      page->changed_timeout = g_timeout_add_full (G_PRIORITY_DEFAULT_IDLE, 250, (GSourceFunc)audio_tags_page_load_tags, page, NULL);
-
-  /* Reset the file attribute */
-  page->file = file;
+  if (page->changed_idle == 0)
+      page->changed_idle = g_idle_add (audio_tags_page_load_tags, page);
 }
 
 
@@ -890,20 +880,20 @@ audio_tags_page_taglib_file_changed (TagLib_File   *taglib_file,
         year = 2006;
 
       /* Set page properties in order to sync with display */
-      g_object_set (G_OBJECT (page), 
-                    "track", (gdouble)track, 
-                    "year", (gdouble)year, 
-                    "artist", artist, 
+      g_object_set (G_OBJECT (page),
+                    "track", (gdouble)track,
+                    "year", (gdouble)year,
+                    "artist", artist,
                     "title", title,
                     "album", album,
                     "comment", comment,
                     "genre", genre,
                     NULL);
-      
+
       /* Free allocated strings */
       taglib_tag_free_strings ();
     }
-  
+
   /* Make page sensitive again */
   gtk_widget_set_sensitive (GTK_WIDGET (page), TRUE);
 }
@@ -914,7 +904,7 @@ static gboolean
 audio_tags_page_activate (AudioTagsPage *page)
 {
   TagLib_Tag *tag;
-    
+
   g_return_val_if_fail (page != NULL || IS_AUDIO_TAGS_PAGE (page), FALSE);
   g_return_val_if_fail (page->file != NULL || THUNARX_IS_FILE_INFO (page->file), FALSE);
   g_return_val_if_fail (page->taglib_file != NULL, FALSE);
@@ -935,7 +925,7 @@ audio_tags_page_activate (AudioTagsPage *page)
         {
           /* Make page insensitive */
           gtk_widget_set_sensitive (GTK_WIDGET (page), FALSE);
-          
+
           /* Store values */
           taglib_tag_set_track (tag, page->track);
           taglib_tag_set_year (tag, page->year);
@@ -952,14 +942,14 @@ audio_tags_page_activate (AudioTagsPage *page)
       /* Free tag strings */
       taglib_tag_free_strings ();
     }
-  
+
   return FALSE;
 }
 
 
 
 static gboolean
-audio_tags_page_info_activate (GtkAction *action, 
+audio_tags_page_info_activate (GtkAction *action,
                                AudioTagsPage *page)
 {
   const TagLib_AudioProperties *properties;
@@ -977,10 +967,10 @@ audio_tags_page_info_activate (GtkAction *action,
 
   gchar         *mimetype;
 
-  ThunarVfsInfo *vfs_info;
-  gchar         *filename;
+  GFileInfo     *fileinfo;
+  const char    *filename;
   gchar         *filesize;
-  
+
   g_return_val_if_fail (page != NULL || IS_AUDIO_TAGS_PAGE (page), FALSE);
   g_return_val_if_fail (page->file != NULL || THUNARX_IS_FILE_INFO (page->file), FALSE);
   g_return_val_if_fail (page->taglib_file != NULL, FALSE);
@@ -1004,12 +994,16 @@ audio_tags_page_info_activate (GtkAction *action,
   bitrate = g_strdup_printf (_("%d KBit/s"), taglib_audioproperties_bitrate (properties));
   samplerate = g_strdup_printf (_("%d Hz"), taglib_audioproperties_samplerate (properties));
   channels = g_strdup_printf ("%d", taglib_audioproperties_channels (properties));
-  
+
   /* Additional information */
   mimetype = thunarx_file_info_get_mime_type (page->file);
-  vfs_info = thunarx_file_info_get_vfs_info (page->file);
-  filename = vfs_info->display_name;
-  filesize = thunar_vfs_humanize_size (vfs_info->size, NULL, 0);
+  fileinfo = thunarx_file_info_get_file_info (page->file);
+  filename = g_file_info_get_display_name (fileinfo);
+#if GLIB_CHECK_VERSION (2, 30, 0)
+  filesize = g_format_size (g_file_info_get_size (fileinfo));
+#else
+  filesize = g_format_size_for_display (g_file_info_get_size (fileinfo));
+#endif
 
   /* Create layout table */
   table = gtk_table_new (7, 2, FALSE);
@@ -1121,19 +1115,20 @@ audio_tags_page_info_activate (GtkAction *action,
   g_free (filesize);
   g_free (mimetype);
 
-  thunar_vfs_info_unref (vfs_info);
-  
+  g_object_unref (fileinfo);
+
   return TRUE;
 }
 
 
 
 static gboolean
-audio_tags_page_load_tags (AudioTagsPage *page)
+audio_tags_page_load_tags (gpointer data)
 {
-  TagLib_File *taglib_file;
-  gchar       *uri;
-  gchar       *filename;
+  AudioTagsPage *page = AUDIO_TAGS_PAGE (data);
+  TagLib_File   *taglib_file;
+  gchar         *uri;
+  gchar         *filename;
 
   g_return_val_if_fail (page != NULL || IS_AUDIO_TAGS_PAGE (page), FALSE);
   g_return_val_if_fail (page->file != NULL || THUNARX_IS_FILE_INFO (page->file), FALSE);
@@ -1154,7 +1149,7 @@ audio_tags_page_load_tags (AudioTagsPage *page)
   g_free (uri);
 
   /* Reset timeout */
-  page->changed_timeout = 0;
+  page->changed_idle = 0;
 
   return FALSE;
 }
@@ -1182,7 +1177,7 @@ audio_tags_page_get_show_save_button (AudioTagsPage *page)
  * audio_tags_page_set_show_save_button:
  * @page : a #AudioTagsPage.
  * @show : a #gboolean flag.
- * 
+ *
  * Creates a save button on the page or removes it.
  **/
 void
@@ -1190,7 +1185,7 @@ audio_tags_page_set_show_save_button (AudioTagsPage   *page,
                                       gboolean         show)
 {
   GtkAction *action;
-  
+
   g_return_if_fail (IS_AUDIO_TAGS_PAGE (page));
   g_return_if_fail (page->table != NULL || GTK_IS_TABLE (page->table));
   g_return_if_fail (page->action_group != NULL || GTK_IS_ACTION_GROUP (page->action_group));
